import requests
import io

from datetime import timedelta
import pandas as pd

from bandl.helper import get_date_range,get_formated_dateframe
from bandl.request import RequestUrl,Curl

#default params for url connection
DEFAULT_TIMEOUT = 5 # seconds
MAX_RETRIES = 2
#default periods
DEFAULT_DAYS = 250

class NasdaqUrls:
    def __init__(self):
        self.STK_DATA_PRE_URL =  r"https://www.nasdaq.com/api/v1/historical/"
        self.date_formats = {"stock_data":"%Y-%m-%d"}

        #historical data header
        self.header = {
            "authority":"www.nasdaq.com",
            "method":"GET",
            "path":"/market-activity/stocks/aapl/historical",
            "scheme":"https",
            "accept":"text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",
            "accept-encoding":"gzip, deflate, br",
            "accept-language":"en-GB,en-US;q=0.9,en;q=0.8",
            "cache-control":"max-age=0",
            "referer":"https://www.nasdaq.com/market-activity/quotes/historical",
            "sec-fetch-dest":"document",
            "sec-fetch-mode":"navigate",
            "sec-fetch-site":"same-origin",
            "sec-fetch-user":"?1",
            "upgrade-insecure-requests":"1",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.138 Safari/537.36"
            }

    def get_data_url(self,symbol,start,end):
        try:
            start =  start.strftime(self.date_formats["stock_data"])
            end = end.strftime(self.date_formats["stock_data"])
            url = self.STK_DATA_PRE_URL + symbol + r"/stocks/" + start + r"/" + end
            return url
        except Exception as err:
            raise Exception("Error occurred in URL constructions ", str(err))

class Nasdaq:
    """Nasdaq class to get data from nasdaq
    """
    def __init__(self,timeout=DEFAULT_TIMEOUT,max_retries=MAX_RETRIES,cloud_mode=False):
        if cloud_mode:
            self.requests = Curl(timeout,max_retries)
        else:
            self.requests = RequestUrl(timeout,max_retries)
        self.nasdaq_url =  NasdaqUrls()

    def __get_data_adjusted(self,dfs,symbol,start=None,end=None,periods=None):
        if periods and (dfs.shape[0] < periods):
            new_periods = periods - dfs.shape[0]
            try:
                s_from = e_till = None
                #if only start, find till today
                if start and (not end):
                    s_from = dfs.index[0] + timedelta(1)
                    e_till = None
                #if not start, can go to past
                elif((end and (not start)) or periods):
                    s_from = None
                    e_till = dfs.index[-1] - timedelta(1)
            except IndexError as err:
                raise Exception("Nasdaq Access error.")
            except Exception as exc:
                raise Exception("Nasdaq data error: ",str(exc))
            try:
                dfs_new = self.get_data(symbol,start = s_from,end = e_till,periods = new_periods)
                dfs = self.__join_dfs(dfs,dfs_new).sort_index(ascending=False)
            except Exception as exc:
                #Small part of data may not be available
                pass
        return dfs

    def __join_dfs(self,join,joiner):
        """will append joiner to join for oi_dfs

        :param join: df which will be appended
        :type join: pandas.DataFrame
        :param joiner: df which we want to append
        :type joiner: pandas.DataFrame
        :return: merged data frame
        :rtype: pandas.DataFrame
        """
        return join.append(joiner)

    def get_data(self,symbol,start=None,end=None,periods=None,dayfirst=False):
        """get_data API to fetch data from nasdaq

        :param symbol: stock symbol
        :type symbol: string
        :param start: start date, defaults to None
        :type start: string, optional
        :param end: end date, defaults to None
        :type end: string, optional
        :param periods: number of days, defaults to None
        :type periods: integer, optional
        :param dayfirst: True if date format is european style DD/MM/YYYY, defaults to False
        :type dayfirst: bool, optional
        :raises ValueError: for invalid inputs
        :raises Exception: incase if no data found
        :return: stock data
        :rtype: pandas.DataFrame
        """
        try:
            #Step1: get the date range
            s_from,e_till = get_date_range(start=start,end=end,periods=periods,dayfirst=dayfirst)

            if s_from > e_till:
                    raise ValueError("End should grater than start.")

            url = self.nasdaq_url.get_data_url(symbol=symbol,start=s_from,end=e_till)
            res = self.requests.get(url,headers=self.nasdaq_url.header)

            try:
                dfs = pd.read_csv(io.StringIO(res.content.decode('utf-8')))
            except Exception as err:
                #increase data range, nasdaq not returning for small set
                if e_till ==  get_formated_dateframe():
                    raise Exception("Nasdaq not retruning data for this date range.\
                                     Please, retry with other date ranges")
                e_till = get_formated_dateframe()
                if (e_till - s_from).days < DEFAULT_DAYS:
                    s_from = e_till - DEFAULT_DAYS
                dfs = self.get_data(symbol,start=s_from,end=e_till)

            dfs.set_index("Date",inplace=True)
            #convert to  datetime
            dfs.index = pd.to_datetime(dfs.index)
            dfs = self.__get_data_adjusted(dfs,symbol,start=start,end=end,periods=periods)
            dfs = dfs.replace('$','',regex=True)
            if not dfs.empty:
                #dfs = dfs.apply(pd.to_numeric)
                return dfs
        except Exception as err:
            raise Exception("Error occurred while getting data :", str(err))


import os
import concurrent.futures
import io
import math

from datetime import datetime,timedelta
import pandas as pd
from requests.exceptions import HTTPError
from bs4 import BeautifulSoup

from bandl.nse_urls import NseUrls
from bandl.helper import get_formated_date,get_date_range
from bandl.request import RequestUrl

#default params for url connection
DEFAULT_TIMEOUT = 5 # seconds
MAX_RETRIES = 2
INDEX_DATA_LIMIT = 99
STOCK_DATA_LIMIT = 240
#default to last three month
PART_OI_DAYS = 22*3

#to disable pandas warning
pd.set_option('mode.chained_assignment', None)

class NseData:
    def __init__(self,timeout=DEFAULT_TIMEOUT,max_retries=MAX_RETRIES):
        self.__nse_urls = NseUrls()
        self.__headers = self.__nse_urls.header
        #create request
        self.__request = RequestUrl(timeout,max_retries)

    def get_indices(self):
        """To get list of NSE indices
        """
        try:
            index_page = self.__request.get(self.__nse_urls.INDEX_URL,headers=self.__headers)
            soup = BeautifulSoup(index_page.text,'lxml')
            table = soup.find("select",{"id":"indexType"})
            indices_data = table.find_all("option")
            indices = [index.get("value") for index in indices_data if "NIFTY" in index.get("value")]

            #lets append india vix as well
            indices.append("INDIA VIX")
            return indices
        except Exception as err:
            raise Exception("Error occurred while getting NSE indices :", str(err))

    def get_oc_exp_dates(self,symbol):

        """get current  available expiry dates

        :raises Exception: NSE connection related
        :return: expiry dates
        :rtype: list
        """
        try:
            base_oc_url = self.__nse_urls.get_option_chain_url(symbol)
            page = self.__request.get(base_oc_url,headers=self.__headers)
            soup = BeautifulSoup(page.text,'lxml')
            table = soup.find("select",{"id":"date"})
            expiry_out = table.find_all("option")
            expiry_dates = [exp_date.get("value") for exp_date in expiry_out][1:]
            return expiry_dates

        except Exception as err:
            raise Exception("something went wrong while reading nse URL :", str(err))

    def get_option_chain_df(self, symbol, expiry_date=None,dayfirst=False):
        """ This function fetches option chain data from NSE and returns in pandas.DataFrame

        :param symbol: stock/index symbol
        :type symbol: string
        :param expiry_date: expiry date (all date formats accepted), defaults to next near
        :type expiry_date: string
        :param dayfirst: True if date format is european style DD/MM/YYYY, defaults to False
        :type dayfirst: bool, optional
        :raises Exception: NSE connection related
        :raises Exception: In html parsing
        :return: option chain
        :rtype: pandas.DataFrame
        """
        try:
            if not expiry_date:
                expiry_date = self.get_oc_exp_dates(symbol)[0]

            oc_url = self.__nse_urls.get_option_chain_url(symbol, expiry_date,dayfirst)
            # If the response was successful, no Exception will be raised
            oc_page = self.__request.get(oc_url, headers = self.__headers)

        except Exception as err:
             raise Exception("Error occurred while connecting NSE :", str(err))
        else:
            try:
                dfs = pd.read_html(oc_page.text)
                return dfs[1]
            except Exception as err:
                raise Exception("Error occurred while reading html :", str(err))

    def __get_file_path(self, file_name, file_path = None, is_use_default_name = True):
        """[summary]

        :param file_name: file name
        :type file_name: string
        :param file_path: file directory or file path , defaults to None
        :type file_path: string, optional
        :param is_use_default_name: to get filename in current timestamp, defaults to True
        :type is_use_default_name: bool, optional
        :return: file path
        :rtype: string
        """
        try:
            if not file_path:
                file_path = os.getcwd()

            if os.path.isfile(file_path):
                if (not is_use_default_name):
                    return file_path
                # if need to use default file path, we get parent path
                else:
                    file_path = os.path.dirname(file_path)

            # datetime object containing current date and time
            now = datetime.now()
            # dd/mm/YY H:M:S
            dt_string = now.strftime("%d_%B_%H_%M")
            file_name = file_name + "_" + dt_string + ".xlsx"

            excel_path = os.path.join(file_path, file_name)
            return excel_path
        except Exception as err:
            print("Error while naming file. Error: ", str(err))

    def get_option_chain_excel(self, symbol, expiry_date=None,dayfirst=False,file_path = None, is_use_default_name = True):
        """Fetches NSE option chain data and returns in the form of excel (.xlsx)

        :param symbol: stock/index symbol
        :type symbol: string
        :param expiry_date: expiry date (all date formats accepted), defaults to next near
        :type expiry_date: string
        :param dayfirst: True if date format is european style DD/MM/YYYY, defaults to False
        :type dayfirst: bool, optional
        :param file_path: file/folder path, defaults to None
        :type file_path: string, optional
        :param is_use_default_name:  to get filename as current timestamp, defaults to True
        :type is_use_default_name: bool, optional
        :raises Exception:  NSE connection related
        """
        try:
            if not expiry_date:
                expiry_date = self.get_oc_exp_dates(symbol)[0]

            df = self.get_option_chain_df(symbol, expiry_date,dayfirst)
            file_name = symbol + "_" + expiry_date
            excel_path = self.__get_file_path(file_name, file_path, is_use_default_name)

            df.to_excel(excel_path, file_name)
        except Exception as err:
            raise Exception("Error occurred while getting excel :", str(err))

    def __join_part_oi_dfs(self,df_join,df_joiner):
        """will append joiner to join for oi_dfs

        :param df_join: Dictionary of participants
        :type df_join: dict
        :param df_joiner: Dictionary of participants
        :type df_joiner: dict
        """
        for client in df_join:
            df_join[client] = self.__join_dfs(df_join[client],df_joiner[client]).sort_index()

    def __join_dfs(self,join,joiner):
        """will append joiner to join for oi_dfs

        :param join: df which will be appended
        :type join: pandas.DataFrame
        :param joiner: df which we want to append
        :type joiner: pandas.DataFrame
        :return: merged data frame
        :rtype: pandas.DataFrame
        """
        return join.append(joiner)

    def get_part_oi_df(self,start=None,end=None,periods=None,dayfirst=False,workers=None):
        """Return dictionary of participants containing data frames

        :param start: start date , defaults to None
        :type start: string, optional
        :param end: end date, defaults to None
        :type end: string, optional
        :param periods: number of days, defaults to None
        :type periods: interger, optional
        :param dayfirst: True if date format is european style DD/MM/YYYY, defaults to False
        :type dayfirst: bool, optional
        :param workers: Number of threads for requesting nse, defaults to None
        :type workers: interger, optional
        :raises Exception: NSE Connection/Request overload
        :return: participant wise open interest
        :rtype: pandas.DataFrame
        """
        try:
            #format date just in case
            if start:
                start = get_formated_date(start,dayfirst=dayfirst)
            if end:
                end = get_formated_date(end,dayfirst=dayfirst)

            #if both none, we set end to today
            if not start and not end:
                end = get_formated_date()
                if not periods:
                    periods = PART_OI_DAYS
            #get urls for these days
            dates = pd.date_range(start=start,end=end, periods=periods,freq='B')
            url_date = [(self.__nse_urls.get_participant_oi_url(date),date) for date in dates]#

            oi_clm = self.__nse_urls.PART_OI_CLM
            #lets preinitialize, better readability
            oi_dfs = {  "Client":pd.DataFrame(columns=oi_clm,index=dates),
                        "DII":pd.DataFrame(columns=oi_clm,index=dates),
                        "FII":pd.DataFrame(columns=oi_clm,index=dates),
                        "Pro":pd.DataFrame(columns=oi_clm,index=dates),
                        "TOTAL":pd.DataFrame(columns=oi_clm,index=dates)
                        }

            if not workers:
                workers = os.cpu_count() * 2

            with concurrent.futures.ThreadPoolExecutor(max_workers=workers) as executor:
                responses = {executor.submit(self.__request.get, url,self.__headers): date for url,date in url_date}
                for res in concurrent.futures.as_completed(responses):
                    date = responses[res]
                    try:
                        csv = res.result()
                    except Exception as exc:
                        #might be holiday
                        pass
                    else:
                        df = pd.read_csv(io.StringIO(csv.content.decode('utf-8')))
                        #drop the first header
                        df_header = df.iloc[0]
                        #is there any implace way?
                        df = df[1:]
                        df.columns = df_header
                        df.set_index('Client Type',inplace=True)
                        #lets us create data frome for all client type
                        oi_dfs['Client'].loc[date] = df.loc['Client']
                        oi_dfs['FII'].loc[date] = df.loc['FII']
                        oi_dfs['DII'].loc[date] = df.loc['DII']
                        oi_dfs['Pro'].loc[date] = df.loc['Pro']
                        oi_dfs['TOTAL'].loc[date] = df.loc['TOTAL']

            if not oi_dfs['Client'].empty:
                #remove nan row
                for client in oi_dfs:
                    oi_dfs[client].dropna(inplace=True)

                #if holiday occurred in business day, lets retrive more data equivalent to holdidays.
                if oi_dfs['Client'].shape[0] < periods:
                    new_periods = periods - oi_dfs['Client'].shape[0]
                    try:
                        #if only start, find till today
                        if start and (not end):
                            s_from = oi_dfs['Client'].index[-1] + timedelta(1)
                            e_till = None
                        #if not start, can go to past
                        elif(end and (not start)):
                            s_from = None
                            e_till = oi_dfs['Client'].index[0] - timedelta(1)
                        #if start and end, no need to change
                        else:
                            return oi_dfs
                    except IndexError as err:
                        raise Exception("NSE Access error.size down/clean cookies to resolve the issue.")
                    except Exception as exc:
                        raise Exception("participant OI error: ",str(exc))

                    oi_dfs_new = self.get_part_oi_df(start = s_from,end = e_till,periods = new_periods)
                    self.__join_part_oi_dfs(oi_dfs,oi_dfs_new)

                return oi_dfs

        except Exception as err:
            raise Exception("Error occurred while getting part_oi :", str(err))

    def __parse_indexdata(self,res_txt,symbol):
        dfs = pd.read_html(res_txt)[0]
        if dfs.shape[0] <2:
            raise Exception("No record found")
        if "NIFTY" in symbol:
            fined_dfs = dfs.iloc[0:]
            fined_dfs.columns = self.__nse_urls.INDEX_DATA_CLM
        elif symbol == "INDIA VIX":
            fined_dfs = dfs.iloc[1:]
            fined_dfs.drop(fined_dfs.index[0],inplace=True)
            fined_dfs.columns = self.__nse_urls.VIX_DATA_CLM
        fined_dfs.drop(fined_dfs.index[-1],inplace=True)
        fined_dfs.set_index("Date",inplace=True)
        return fined_dfs

    def __get_datarange_intv(self,start,end,intv):
        diff = math.ceil((end  - start).days / intv)
        date_ranges = []
        curr_start = prev_start = start
        for i in range(diff):
            curr_start =  (start + timedelta(intv * i))
            if i !=0:
                start_ = prev_start
                end_ = curr_start - timedelta(1)
                date_ranges.append((start_,end_))
            prev_start = curr_start
        date_ranges.append((curr_start,end))
        return date_ranges

    def __get_data_adjusted(self,dfs,symbol,series="EQ",start=None,end=None,periods=None):
        if periods and (dfs.shape[0] < periods):
            new_periods = periods - dfs.shape[0]
            try:
                s_from = e_till = None
                #if only start, find till today
                if start and (not end):
                    s_from = dfs.index[0] + timedelta(1)
                    e_till = None
                #if not start, can go to past
                elif((end and (not start)) or periods):
                    s_from = None
                    e_till = dfs.index[-1] - timedelta(1)
            except IndexError as err:
                raise Exception("NSE Access error.")
            except Exception as exc:
                raise Exception("Stock data error: ",str(exc))
            try:
                dfs_new = self.get_data(symbol,series,start = s_from,end = e_till,periods = new_periods)
                dfs = self.__join_dfs(dfs,dfs_new).sort_index(ascending=False)
            except Exception as exc:
                #data may not be available
                pass
        return dfs


    def get_data(self,symbol,series="EQ",start=None,end=None,periods=None,dayfirst=False):
        """To get NSE stock data

        :param symbol: stock/index symbol
        :type symbol: string
        :param series: segment, defaults to "EQ"
        :type series: string, optional
        :param start: start date, defaults to None
        :type start: string, optional
        :param end: end date, defaults to None
        :type end: string, optional
        :param periods: number of days, defaults to None
        :type periods: interger, optional
        :param dayfirst: True if date format is european style DD/MM/YYYY, defaults to False
        :type dayfirst: bool, optional
        :raises Exception: NSE Connection Related
        :return: stock data
        :rtype: pandas.DataFrame
        """
        try:

            #Step1: get the date range
            s_from,e_till = get_date_range(start=start,end=end,periods=periods,dayfirst=dayfirst)

            if s_from > e_till:
                raise ValueError("End should grater than start.")

            data_limit = None
            if self.__nse_urls.is_index(symbol):
                data_limit = INDEX_DATA_LIMIT
            else:
                data_limit = STOCK_DATA_LIMIT

            data_days = e_till - s_from
            if (data_days.days) > data_limit:
                date_ranges = self.__get_datarange_intv(s_from,e_till,data_limit)
                workers = len(date_ranges)
                with concurrent.futures.ThreadPoolExecutor(max_workers=workers) as executor:
                    responses = [executor.submit(self.get_data, symbol=symbol,start=start_,end=end_,dayfirst=dayfirst)\
                                    for start_,end_ in date_ranges]
                    dfs = []
                    for res in concurrent.futures.as_completed(responses):
                        try:
                            df = res.result()
                            dfs.append(df)
                        except Exception as exc:
                            #might be holiday/no record
                            pass
                    all_dfs = pd.concat(dfs).sort_index(ascending=False)
                    adjusted_dfs = self.__get_data_adjusted(all_dfs,symbol,start=start,end=end,periods=periods)
                    return adjusted_dfs

            data_url = self.__nse_urls.get_stock_data_url\
                                                        (
                                                        symbol,series=series,start=s_from,
                                                        end=e_till
                                                        )

            csv = self.__request.get(data_url,headers=self.__headers)

            #if it is index, wee need to read table
            # Why the heck, We are doing so much handling? Is there any other way?
            # Suggestions are welcome. ping me on github
            if self.__nse_urls.is_index(symbol):
                dfs = self.__parse_indexdata(csv.text,symbol)
            else:
                dfs = pd.read_csv(io.StringIO(csv.content.decode('utf-8')))
                dfs.set_index('Date ',inplace=True)
            # Converting the index as date
            dfs.index = pd.to_datetime(dfs.index)
            dfs = self.__get_data_adjusted(dfs,symbol,start=start,end=end,periods=periods)
            return dfs

        except Exception as err:
            raise Exception("Error occurred while fetching stock data :", str(err))


import subprocess

import requests
from requests.adapters import HTTPAdapter
from urllib3.util import Retry

DEFAULT_TIMEOUT = 5 # seconds
MAX_RETRIES = 2

#class TimeoutHTTPAdapter credit : https://github.com/psf/requests/issues/3070#issuecomment-205070203
class TimeoutHTTPAdapter(HTTPAdapter):
    def __init__(self, *args, **kwargs):
        self.timeout = DEFAULT_TIMEOUT
        if "timeout" in kwargs:
            self.timeout = kwargs["timeout"]
            del kwargs["timeout"]
        super().__init__(*args, **kwargs)

    def send(self, request, **kwargs):
        timeout = kwargs.get("timeout")
        if timeout is None:
            kwargs["timeout"] = self.timeout
        return super().send(request, **kwargs)

# If hitting curl with subprocess
class DummyResponse:
    def __init__(self,txt):
        self.text = None
        self.content = None
        if txt:
            try:
                self.text = txt.decode('utf-8')
            except UnicodeDecodeError:
                self.text = txt.reason.decode('iso-8859-1')
            self.content = txt
            self.status_code = 200
        else:
            raise Exception("Could not connect to host")

class Curl:
    def __init__(self,timeout=DEFAULT_TIMEOUT,max_retries=MAX_RETRIES):
        self.retry = max_retries
        self.timeout = timeout

    def __curl_cmd(self,url,headers):
        cmd = 'curl --connect-timeout ' + str(self.timeout) + ' --retry '+ str(self.retry) + ' "' + url + '" '
        query = " -H ".join(['"%s:%s"'%(key,value) for key,value in headers.items()])
        curl_cmd = cmd + "-H " + query + ' --compressed'
        return curl_cmd

    def get(self,url,headers):
        curl_cmd = self.__curl_cmd(url,headers)
        content = subprocess.Popen(curl_cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE,shell=True)
        txt, _ = content.communicate()
        try:
            response= DummyResponse(txt)
            return response
        except Exception as err:
            raise Exception(str(err))


class RequestUrl:
    def __init__(self,timeout=DEFAULT_TIMEOUT,max_retries=MAX_RETRIES):
        self.session = self.get_session(timeout=timeout,max_retries=max_retries)


    def get_session(self,timeout=DEFAULT_TIMEOUT,max_retries=MAX_RETRIES):
        #backoff_factor allows us to change how long the processes will sleep between failed requests
        retries = Retry(
                    total=max_retries,
                    backoff_factor=1,
                    status_forcelist=[429, 500, 502, 503, 504],
                    )
        adapter = TimeoutHTTPAdapter(max_retries=retries ,timeout=timeout)
        session = requests.Session()
        session.mount("https://", adapter)
        session.mount("http://", adapter)
        return session

    def get(self,*args,**kwargs):
        try:
            page = self.session.get(*args, **kwargs)
            # If the response was successful, no Exception will be raised
            page.raise_for_status()
            return page
        except requests.HTTPError as http_err:
            raise Exception("HTTP error occurred while fetching url :", str(http_err.response.content))

    def post(self,*args,**kwargs):
        try:
            page = self.session.post(*args, **kwargs)
            # If the response was successful, no Exception will be raised
            page.raise_for_status()
            return page
        except requests.HTTPError as http_err:
            raise Exception("HTTP error occurred while fetching url :", str(http_err.response.content))




from datetime import date as dt
import pandas as pd
import requests

import bandl.common

#default periods
DEFAULT_DAYS = 250


def is_ind_index(symbol):
    is_it =  symbol in bandl.common.IND_INDICES
    return is_it

def get_formated_date(date=None,format=None,dayfirst=False):
    """string date to format date
    """
    try:
        if not date:
            date = dt.today()
        date_time = pd.to_datetime(date,dayfirst=dayfirst)
        if not format:
            format='%m/%d/%Y'
            format += ' %H:%M:%S'

        return date_time.strftime(format)

    except Exception as err:
        raise Exception("Error occurred while formatting date, Error: ",str(err))

def get_formated_dateframe(date=None,format=None,dayfirst=False):
    return pd.to_datetime(get_formated_date(date,format,dayfirst),format=format)

def get_date_offset(periods=None,start=None,end=None,freq="B"):
    #use to get start date and end date
    if start:
        if not periods:
            return get_formated_date()
        else:
            return pd.date_range(start=start,end=end,periods=periods,freq=freq)[-1]
    elif end:
       return pd.date_range(start=start,end=end,periods=periods,freq=freq)[0]
    else:
        raise ValueError("start/end , one should be None")

def get_date_range(start=None,end=None,periods=None,format=None,dayfirst=False,freq="B"):
    #Step 1: format date
    if start:
        start = get_formated_dateframe(start,dayfirst=dayfirst)
    if end:
        end = get_formated_dateframe(end,dayfirst=dayfirst)

    #Step 2: date range with periods
    if (not periods) and (not start):
        periods = DEFAULT_DAYS
    #if only start, find till today
    if start and (not end):
        s_from = start
        e_till = get_date_offset(start=start,periods=periods)#s_from + pd.offsets.BDay(periods)
    #if not start, go to past
    elif(end and (not start)):
        s_from = get_date_offset(end=end,periods=periods)#e_till - pd.offsets.BDay(periods)
        e_till = end
    #if start and end, no need to change
    elif(start and end):
        s_from = start
        e_till = end
    # if no stat/end and periods given, we get last 1 years of data
    else:
        e_till = get_formated_dateframe()
        s_from = get_date_offset(end=e_till,periods=periods,freq=freq)

    #Step 3: Format to input date format
    s_from = get_formated_dateframe(date=s_from,format=format)
    e_till = get_formated_dateframe(date=e_till,format=format)

    return s_from,e_till

def get_data_resample(dfs,time):
    dfs.columns = dfs.columns.str.title()
    ohlc_dict = {'Open':'first', 'High':'max', 'Low':'min', 'Close': 'last','Volume':'sum'}
    return dfs.resample(time,convention="end").agg(ohlc_dict).dropna()


from setuptools import setup,find_packages

with open("README.md", 'r') as f:
    README = f.read()

setup(
   name='bandl',
   version='0.0.1',
   description='Utilities for the analysis of financial data',
   license="MIT",
   long_description=README,
   long_description_content_type='text/markdown',
   author='stolgo Developers',
   author_email='stockalgos@gmail.com',
   project_urls={
          "Organization":"http://www.stolgo.com",
          "Source":"https://github.com/chiranjeevg/bandl/",
          "Tracker":"https://github.com/chiranjeevg/bandl/issues"
          },
   packages=find_packages('lib'),
   package_dir = {'':'lib'},
   include_package_data=True,
   install_requires=[
            'requests',
            'pandas',
            'datetime',
            'openpyxl',
            'futures',
            'beautifulsoup4',
            'lxml'],
    classifiers=[
      'Development Status :: 5 - Production/Stable',
      'Intended Audience :: Developers',
      'Topic :: Software Development :: Build Tools',
      'License :: OSI Approved :: MIT License',
      'Programming Language :: Python :: 3',
      'Programming Language :: Python :: 3.5',
      'Programming Language :: Python :: 3.6',
      'Programming Language :: Python :: 3.7',
      'Programming Language :: Python :: 3.8',
        ],
    download_url = "https://github.com/chiranjeevg/bandl/archive/v0.0.1.tar.gz",
    keywords = ['data', 'NSE', 'STOCK','FINANCE',"DERIVATIVE","API"]
)